/* Generated by re2c 0.9.11.rc1 on Sat Feb 25 13:34:43 2006 */
#line 1 "src/lexer.re"
/* Copyright 2005-2006 Garrett Rooney.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <stdlib.h> /* for abort */

#include "lexer.h"

#include <apr_strings.h>

#define BSIZE          8192
#define MAX_TEXT_BYTES 1024

struct etl_lexer_t {
  apr_bucket_brigade *bb;  /* the brigade we're reading from */
  apr_bucket_brigade *tbb; /* a temp brigade for reading lines */

  /* bookkeeping data for the re2c lexer */
  char *buffer;
  char *cursor;
  char *limit;
  char *marker;
  char *tstart;

  /* bookkeeping data for the stored up text */
  char text_buffer[MAX_TEXT_BYTES];
  int ntext_bytes;
  int saved_bytes;

  int pending_token; /* next token after a text node, saved to return later */

  int eof;          /* have we hit the end of the input? */
  int in_directive; /* are we inside a etl directive? */
};

etl_lexer_t *
etl_lexer_create (apr_bucket_brigade *bb, apr_pool_t *pool)
{
  etl_lexer_t *l = apr_pcalloc (pool, sizeof (*l));

  l->bb = bb;
  l->tbb = apr_brigade_create (pool, bb->bucket_alloc);
  l->buffer = apr_pcalloc (pool, BSIZE);

  return l;
}

static apr_status_t
fill (etl_lexer_t *l)
{
  if (l->cursor == l->limit)
    {
      apr_status_t err = apr_brigade_split_line (l->tbb,
                                                 l->bb,
                                                 APR_BLOCK_READ,
                                                 BSIZE - 1);
      if (err)
        return err;
      else
        {
          apr_size_t len = BSIZE;

          apr_brigade_flatten (l->tbb, l->buffer, &len);

          if (len == 0)
            {
              l->eof = 1;
              l->buffer[0] = '\n';
              l->buffer[1] = '\0';
              l->limit = l->buffer;
            }
          else
            {
              l->limit = l->buffer + len;
            }

          apr_brigade_cleanup (l->tbb);
        }

      l->cursor = l->marker = l->tstart = l->buffer;
    }

  return APR_SUCCESS;
}

/* Defines needed for re2c. */
#define  YYCTYPE   char
#define  YYCURSOR  l->cursor
#define  YYLIMIT   l->limit
#define  YYMARKER  l->marker
#define  YYFILL(n) { apr_status_t rv = fill(l); if (rv) return rv; }

/* A little shorthand... */
#define RET(i) do {                                   \
                 if (l->ntext_bytes)                  \
                   {                                  \
                     l->pending_token = i;            \
                     *t = ETL_TOK_TEXT;               \
                     l->saved_bytes = l->ntext_bytes; \
                     l->ntext_bytes = 0;              \
                   }                                  \
                 else                                 \
                   {                                  \
                     *t = i;                          \
                   }                                  \
                 return APR_SUCCESS;                  \
               } while (0)

apr_status_t
etl_lexer_scan (etl_token_t *t, etl_lexer_t *l, apr_pool_t *pool)
{
  if (l->pending_token)
    {
      *t = l->pending_token;

      l->pending_token = 0;

      return APR_SUCCESS;
    }

start:
  l->tstart = l->cursor;

  if (! l->in_directive)
    {
      
#line 141 "<stdout>"
{
	YYCTYPE yych;
	goto yy0;
	++YYCURSOR;
yy0:
	if((YYLIMIT - YYCURSOR) < 2) YYFILL(2);
	yych = *YYCURSOR;
	switch(yych){
	case 0x0A:	goto yy4;
	case '[':	goto yy2;
	default:	goto yy6;
	}
yy2:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case '%':	goto yy7;
	default:	goto yy3;
	}
yy3:
#line 156 "src/lexer.re"
{
                      text:
                        l->text_buffer[l->ntext_bytes] = *(YYCURSOR - 1);
                        l->ntext_bytes++;

                        if (l->ntext_bytes == MAX_TEXT_BYTES)
                          {
                            *t = ETL_TOK_TEXT;
                            l->saved_bytes = l->ntext_bytes;
                            l->ntext_bytes = 0;
                            return APR_SUCCESS;
                          }
                        else
                          goto start;
                      }
#line 176 "<stdout>"
yy4:	++YYCURSOR;
	goto yy5;
yy5:
#line 149 "src/lexer.re"
{
                        if (l->eof)
                          RET(ETL_TOK_EOI);
                        else
                          goto text;
                      }
#line 187 "<stdout>"
yy6:	yych = *++YYCURSOR;
	goto yy3;
yy7:	++YYCURSOR;
	goto yy8;
yy8:
#line 144 "src/lexer.re"
{
                        l->in_directive = 1;
                        RET(ETL_TOK_START_DIR);
                      }
#line 198 "<stdout>"
}
#line 171 "src/lexer.re"

    }
  else
    {
      
#line 206 "<stdout>"
{
	YYCTYPE yych;
	goto yy9;
	++YYCURSOR;
yy9:
	if((YYLIMIT - YYCURSOR) < 7) YYFILL(7);
	yych = *YYCURSOR;
	switch(yych){
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:	case 0x0B:
	case 0x0C:	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:	case '!':
	case '"':
	case '#':
	case '$':	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':	case '}':
	case '~':
	case 0x7F:	goto yy26;
	case 0x09:	case 0x0D:	case ' ':	goto yy21;
	case 0x0A:	goto yy24;
	case '%':	goto yy12;
	case 'e':	goto yy16;
	case 'f':	goto yy17;
	case 'i':	goto yy14;
	case 'p':	goto yy18;
	case 'u':	goto yy15;
	case '|':	goto yy19;
	default:	goto yy23;
	}
yy11:
#line 190 "src/lexer.re"
{ RET(ETL_TOK_VAR); }
#line 288 "<stdout>"
yy12:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case ']':	goto yy54;
	default:	goto yy13;
	}
yy13:
#line 197 "src/lexer.re"
{
                        /* We don't understand it, so... */
                        return APR_EINVAL;
                      }
#line 300 "<stdout>"
yy14:	yych = *++YYCURSOR;
	switch(yych){
	case 'f':	goto yy50;
	case 'n':	goto yy52;
	default:	goto yy28;
	}
yy15:	yych = *++YYCURSOR;
	switch(yych){
	case 'n':	goto yy44;
	default:	goto yy28;
	}
yy16:	yych = *++YYCURSOR;
	switch(yych){
	case 'l':	goto yy37;
	case 'n':	goto yy38;
	default:	goto yy28;
	}
yy17:	yych = *++YYCURSOR;
	switch(yych){
	case 'o':	goto yy34;
	default:	goto yy28;
	}
yy18:	yych = *++YYCURSOR;
	switch(yych){
	case 'r':	goto yy29;
	default:	goto yy28;
	}
yy19:	++YYCURSOR;
	goto yy20;
yy20:
#line 188 "src/lexer.re"
{ RET(ETL_TOK_PIPE); }
#line 333 "<stdout>"
yy21:	++YYCURSOR;
	goto yy22;
yy22:
#line 189 "src/lexer.re"
{ goto start; }
#line 339 "<stdout>"
yy23:	yych = *++YYCURSOR;
	goto yy28;
yy24:	++YYCURSOR;
	goto yy25;
yy25:
#line 191 "src/lexer.re"
{
                        if (l->eof)
                          RET(ETL_TOK_EOI);
                        else
                          goto start;
                      }
#line 352 "<stdout>"
yy26:	yych = *++YYCURSOR;
	goto yy13;
yy27:	++YYCURSOR;
	if(YYLIMIT == YYCURSOR) YYFILL(1);
	yych = *YYCURSOR;
	goto yy28;
yy28:	switch(yych){
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy11;
	default:	goto yy27;
	}
yy29:	yych = *++YYCURSOR;
	switch(yych){
	case 'i':	goto yy30;
	default:	goto yy28;
	}
yy30:	yych = *++YYCURSOR;
	switch(yych){
	case 'n':	goto yy31;
	default:	goto yy28;
	}
yy31:	yych = *++YYCURSOR;
	switch(yych){
	case 't':	goto yy32;
	default:	goto yy28;
	}
yy32:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy33;
	default:	goto yy27;
	}
yy33:
#line 187 "src/lexer.re"
{ RET(ETL_TOK_PRINT); }
#line 524 "<stdout>"
yy34:	yych = *++YYCURSOR;
	switch(yych){
	case 'r':	goto yy35;
	default:	goto yy28;
	}
yy35:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy36;
	default:	goto yy27;
	}
yy36:
#line 185 "src/lexer.re"
{ RET(ETL_TOK_FOR); }
#line 607 "<stdout>"
yy37:	yych = *++YYCURSOR;
	switch(yych){
	case 's':	goto yy41;
	default:	goto yy28;
	}
yy38:	yych = *++YYCURSOR;
	switch(yych){
	case 'd':	goto yy39;
	default:	goto yy28;
	}
yy39:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy40;
	default:	goto yy27;
	}
yy40:
#line 184 "src/lexer.re"
{ RET(ETL_TOK_END); }
#line 695 "<stdout>"
yy41:	yych = *++YYCURSOR;
	switch(yych){
	case 'e':	goto yy42;
	default:	goto yy28;
	}
yy42:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy43;
	default:	goto yy27;
	}
yy43:
#line 186 "src/lexer.re"
{ RET(ETL_TOK_ELSE); }
#line 778 "<stdout>"
yy44:	yych = *++YYCURSOR;
	switch(yych){
	case 'l':	goto yy45;
	default:	goto yy28;
	}
yy45:	yych = *++YYCURSOR;
	switch(yych){
	case 'e':	goto yy46;
	default:	goto yy28;
	}
yy46:	yych = *++YYCURSOR;
	switch(yych){
	case 's':	goto yy47;
	default:	goto yy28;
	}
yy47:	yych = *++YYCURSOR;
	switch(yych){
	case 's':	goto yy48;
	default:	goto yy28;
	}
yy48:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy49;
	default:	goto yy27;
	}
yy49:
#line 183 "src/lexer.re"
{ RET(ETL_TOK_UNLESS); }
#line 876 "<stdout>"
yy50:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy51;
	default:	goto yy27;
	}
yy51:
#line 181 "src/lexer.re"
{ RET(ETL_TOK_IF); }
#line 954 "<stdout>"
yy52:	++YYCURSOR;
	switch((yych = *YYCURSOR)) {
	case 0x00:
	case 0x01:
	case 0x02:
	case 0x03:
	case 0x04:
	case 0x05:
	case 0x06:
	case 0x07:
	case 0x08:
	case 0x09:
	case 0x0A:
	case 0x0B:
	case 0x0C:
	case 0x0D:
	case 0x0E:
	case 0x0F:
	case 0x10:
	case 0x11:
	case 0x12:
	case 0x13:
	case 0x14:
	case 0x15:
	case 0x16:
	case 0x17:
	case 0x18:
	case 0x19:
	case 0x1A:
	case 0x1B:
	case 0x1C:
	case 0x1D:
	case 0x1E:
	case 0x1F:
	case ' ':
	case '!':
	case '"':
	case '#':
	case '$':
	case '%':
	case '&':
	case '\'':
	case '(':
	case ')':
	case '*':
	case '+':	case '-':	case '/':
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
	case '8':
	case '9':
	case ':':
	case ';':
	case '<':
	case '=':
	case '>':
	case '?':
	case '@':	case '[':
	case '\\':
	case ']':
	case '^':
	case '_':
	case '`':	case '{':
	case '|':
	case '}':
	case '~':
	case 0x7F:	goto yy53;
	default:	goto yy27;
	}
yy53:
#line 182 "src/lexer.re"
{ RET(ETL_TOK_IN); }
#line 1032 "<stdout>"
yy54:	++YYCURSOR;
	goto yy55;
yy55:
#line 177 "src/lexer.re"
{
                        l->in_directive = 0;
                        RET(ETL_TOK_END_DIR);
                      }
#line 1041 "<stdout>"
}
#line 201 "src/lexer.re"

    }

  abort();
}

char *
etl_lexer_text (etl_lexer_t *lexer, apr_pool_t *pool)
{
  if (lexer->saved_bytes)
    {
      char *str = apr_pcalloc (pool, lexer->saved_bytes + 1);

      memcpy (str, lexer->text_buffer, lexer->saved_bytes);

      lexer->saved_bytes = 0;

      return str;
    }
  else
    {
      apr_size_t len = lexer->cursor - lexer->tstart;

      char *str = apr_pcalloc (pool, len + 1);

      memcpy (str, lexer->tstart, len);

      return str;
    }
}
